from conversion_helpers import model_to_dict, dict_to_base_type, base_converter
from django.db.models.fields.related import ForeignKey, ManyToManyField

class BaseFancy(object):
    def __init__(self, a_dict):
        self.__dict__ = dict_to_base_type(a_dict, self)
        #self.__dict__ = dict_to_base_type(a_dict, self, base_converter)

    def __eq__(self, other):
        if not other.__dict__ == self.__dict__:
            print "this: " + str(self.__dict__)
            print "is not equal to this: " + str(other.__dict__)

        return other.__dict__ == self.__dict__

    def __ne__(self, other):
        return not self.__eq__(other.__dict__)

class FancyHash(BaseFancy):
    pass

class FancyModel(BaseFancy):
    pass

class FancyModel(BaseFancy):
    def __init__(self, django_model, depth=1):
        temp_dict = {}

        for field in django_model._meta.local_fields:
            field_value = getattr(django_model, field.name)

            if isinstance(field, ForeignKey) and field_value is not None:
                if depth > 0:
                    temp_dict[field.name] = self.__class__(field_value, depth - 1)
            else:
                temp_dict[field.name] = field_value

        if depth > 0:
            for field in django_model._meta.local_many_to_many:
                field_list = []
                for m2m_item in getattr(django_model, field.name).all():
                    field_list.append(self.__class__(m2m_item, depth -1))
                temp_dict[field.name] = field_list

        self.__dict__.update(temp_dict)


    #def __init__(self, a_dict):
    #    self.__dict__ = dict_to_base_type(a_dict, self)
    #def __init__(self, convert_object):
    #    super(FancyModel, self).__init__(convert_object)
    #    temp_dict = model_to_dict(django_model, depth)
    #    base_type_dict = dict_to_base_type(temp_dict, FanyModel)
    #    print base_type_dict
    #    self.__dict__ = base_type_dict



from immutable import ImmutableHash, ImmutableModel
from fancy import FancyHash, FancyModel
from conversion_helpers import model_to_dict, dict_to_base_type

def create_fancy_model(django_model, depth=1):
    #temp_dict = model_to_dict(django_model, depth)
    #base_type_dict = dict_to_base_type(temp_dict, FancyModel)

    #return FancyModel(temp_dict)
    return FancyModel(django_model, depth)

def create_immutable_model(django_model, depth=1):
    #temp_dict = model_to_dict(django_model, depth)
    #base_type_dict = dict_to_base_type(temp_dict, ImmutableModel)

    #return ImmutableModel(temp_dict)
    return ImmutableModel(django_model, depth)




from changeless.types import FancyModel, ImmutableModel

def fancy_list(*args, **kwargs):
    depth = 1
    if kwargs.has_key('depth'):
        depth = kwargs['depth']

    def wrapped(db_function):
        def create_fancy():
            return lambda: [ FancyModel(db_entry, depth) for db_entry in db_function() ]
        return create_fancy()

    if len(args) == 1 and callable(args[0]):
        return wrapped(args[0])
    else:
        return wrapped

def immutable_list(*args, **kwargs):
    depth = 1
    if kwargs.has_key('depth'):
        depth = kwargs['depth']

    def wrapped(db_function):
        def create_immutable():
            return lambda: [ ImmutableModel(db_entry, depth) for db_entry in db_function() ]

        return create_immutable()

    if len(args) == 1 and callable(args[0]):
        return wrapped(args[0])
    else:
        return wrapped

def fancy_gen(*args, **kwargs):
    depth = 1
    if kwargs.has_key('depth'):
        depth = kwargs['depth']

    def wrapped(db_function):

        def create_fancy():
            return lambda: _fancy_gen( db_function(), depth )
        return create_fancy()

    if len(args) == 1 and callable(args[0]):
        return wrapped(args[0])
    else:
        return wrapped


def immutable_gen(*args, **kwargs):
    depth = 1
    if kwargs.has_key('depth'):
        depth = kwargs['depth']

    def wrapped(db_function):
        def create_immutable():
            return lambda: _immutable_gen( db_function(), depth )

        return create_immutable()

    if len(args) == 1 and callable(args[0]):
        return wrapped(args[0])
    else:
        return wrapped

def _immutable_gen(query_set, depth):
    for db_entry in query_set:
        yield FancyModel(db_entry, depth)

def _fancy_gen(query_set, depth):
    for db_entry in query_set:
        yield FancyModel(db_entry, depth)




from changeless.types import ImmutableHash, ImmutableModel, FancyHash, FancyModel

'''imports used for testing'''
import unittest
import os
from changeless.types.immutable import BaseImmutable
from changeless.types.fancy import BaseFancy
import changeless.types as types

'''for django model conversion testing'''
from changeless.test.decorators import reset_db

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

print "loading testing fill"
from django.db.models.fields.related import ForeignKey, ManyToManyField
from django.contrib.auth.models import User
from django.test.simple import DjangoTestSuiteRunner

from changeless.test.myapp.models import Library, Book, Address

from test_helpers import load_fixtures

class TestFancy(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_runner = DjangoTestSuiteRunner(interactive=False, verbosity=1)
        test_db = test_runner.setup_databases()
        load_fixtures()

    def test_fancy_hash_is_fancy_type(self):
        obj = FancyHash({'name': 'test_obj', 'attr1':'attr1_value'})

        self.assertIsInstance(obj, BaseFancy)
        self.assertIsInstance(obj, FancyHash)

    #@reset_db
    def test_fancy_model_is_fancy_type(self):
        obj_model = User.objects.get(username="me")

        print "username in fancy check"
        print obj_model.username

        obj = types.create_fancy_model(obj_model)

        self.assertIsInstance(obj, BaseFancy)
        self.assertIsInstance(obj, FancyModel)

    def test_converts_attributes(self):
        my_instance = FancyHash({"thing_one":"value one", "thing_two":"value two"})

        self.assertEqual(my_instance.thing_one, "value one")
        self.assertEqual(my_instance.thing_two, "value two")

    def test_converts_nested_attribute(self):
        my_instance = FancyHash({"thing_one":"value one", "thing_two":{"name":"thing_two"}})

        self.assertEqual(my_instance.thing_two.name, "thing_two")

    def test_converts_lists(self):
        my_instance = FancyHash({"thing_one":"value one", "a_list":[{"name":"thing_three"},{"name": "thing_four"} ]} )

        self.assertEqual(my_instance.a_list[0].name, "thing_three")
        self.assertEqual(my_instance.a_list[1].name, "thing_four")

    def test_nested_dicts(self):
        my_instance = FancyHash({"thing_one":{"name":"one", "attributes":{"test_attr":"two"}}})
        self.assertEqual(my_instance.thing_one.name, "one")
        self.assertEqual(my_instance.thing_one.attributes.test_attr, "two")

    def test_fancy_model_base_attributes_are_properites(self):
        fancy_model = types.create_fancy_model(User.objects.get(username="me"))
        self.assertEqual(fancy_model.username, "me")
        self.assertEqual(fancy_model.email, "me@aol.com")

    def test_fancy_model_finds_basic_relationship(self):
        fancy_model = types.create_fancy_model(Book.objects.get(title="A Tale of Two Cities"))

        self.assertEqual(fancy_model.author.username, "john")
        self.assertEqual(fancy_model.author.email, "lennon@thebeatles.com")


    def test_fancy_model_default_is_one_relationship_level_deep(self):
        fancy_model = types.create_fancy_model(Book.objects.get(title="A Tale of Two Cities"))

        with self.assertRaises(AttributeError):
            address = fancy_model.location.address

    def test_fancy_model_level_two_attributes(self):
        fancy_model = types.create_fancy_model(Book.objects.get(title="A Tale of Two Cities"), depth=2)

        self.assertEqual(fancy_model.location.address.street_address, "THE place")

    def test_fancy_model_many_to_many_relationships_is_list(self):
        fancy_model = types.create_fancy_model(Book.objects.get(title="A Tale of Two Cities"))

        self.assertIsInstance(fancy_model.readers, list)

    def test_fancy_model_many_to_many_relationships_is_list_with_correct_readers(self):
        fancy_model = types.create_fancy_model(Book.objects.get(title="A Tale of Two Cities"))

        self.assertEqual(len(fancy_model.readers), 2)

class TestImmutable(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        test_runner = DjangoTestSuiteRunner(interactive=False, verbosity=1)
        test_db = test_runner.setup_databases()
        load_fixtures()

    def test_immutable_hash_is_immutable_type(self):
        is_immutable = True;
        obj = ImmutableHash({'name': 'test_obj', 'attr1':'attr1_value'})

        '''checking inheritance'''
        '''and terrible things don't happen during init'''
        self.assertIsInstance(obj, BaseFancy)
        self.assertIsInstance(obj, BaseImmutable)
        self.assertIsInstance(obj, ImmutableHash)

    def test_immutable_hash_is_immutable(self):
        is_immutable = False;
        obj = ImmutableHash({'name': 'test_obj', 'attr1':'attr1_value'})
        with self.assertRaises(Exception) as cm:
            obj.name = "new name"

    #@reset_db
    def test_immutable_model_is_immutable_type(self):
        obj_model = User.objects.get(username="me")

        obj = types.create_immutable_model(obj_model)

        self.assertIsInstance(obj, ImmutableModel)

    def test_immutable_model_base_attributes_are_properites(self):
        fancy_model = ImmutableModel(User.objects.get(username="me"))

        self.assertEqual(fancy_model.username, "me")
        self.assertEqual(fancy_model.email, "me@aol.com")

    def test_immutable_model_finds_basic_relationship(self):
        fancy_model = ImmutableModel(Book.objects.get(title="A Tale of Two Cities"))

        self.assertEqual(fancy_model.author.username, "john")
        self.assertEqual(fancy_model.author.email, "lennon@thebeatles.com")


    def test_immutable_model_default_is_one_relationship_level_deep(self):
        fancy_model = ImmutableModel(Book.objects.get(title="A Tale of Two Cities"))

        with self.assertRaises(AttributeError):
            address = fancy_model.location.address

    def test_immutable_model_level_two_attributes(self):
        fancy_model = ImmutableModel(Book.objects.get(title="A Tale of Two Cities"), depth=2)

        self.assertEqual(fancy_model.location.address.street_address, "THE place")

    def test_immutable_model_many_to_many_relationships_is_list(self):
        fancy_model = ImmutableModel(Book.objects.get(title="A Tale of Two Cities"))

        self.assertIsInstance(fancy_model.readers, list)

    def test_immutable_model_many_to_many_relationships_is_list_with_correct_readers(self):
        immutable_model = ImmutableModel(Book.objects.get(title="A Tale of Two Cities"))

        self.assertEqual(len(immutable_model .readers), 2)






from changeless.types.immutable import ImmutableModel
from changeless.types.fancy import FancyHash, BaseFancy
from changeless.decorators import fancy_gen
import json
import datetime
import logging

def _generatorType():
    @fancy_gen
    def generator_type_function():
        pass
    generator_obj = generator_type_function()
    return type(generator_obj)

GeneratorType = _generatorType()

'''return dictonary object of object given keys minus ignore keys'''
def _sub_dict(object, keys, ignore=[]):
    return_dict = {}

    for key, value in object.__dict__.items():
        if key in keys and key not in ignore:
            return_dict[key] = value

'''return recursive (dictonaries within dictonaries) union of dictonary object.  object and sec_object must be dicts '''
def _r_sub_dict(object, second_obj):
    #todo: add checking for lists and convert immutable models contained within them.
    return_dict = {}
    for key,value in object.items():
        if key in second_obj.keys():
            if isinstance(value, dict) and isinstance(second_obj[key],dict):
                return_dict[key] = _r_sub_dict(value, second_obj[key])
            else:
                return_dict[key] = value

    return return_dict

def _convert_list_of_fancy_hashes(a_list):
    r_immutable_list = []
    for immutable_obj in a_list:
        r_immutable_list.append(to_dict(immutable_obj))
    return r_immutable_list

'''convert all nested immutables into dict.  use im comparison, or kept public for you in cases of pretty printing'''
def to_dict(obj):
    new_dict = {}
    if isinstance(obj, list) or isinstance(obj, GeneratorType):
        return [ to_dict(dict_obj) for dict_obj in obj ]
    else: 
        for key, value in obj.__dict__.items():
            #logging.debug( type(value)  )
            if isinstance(value, BaseFancy) :
                new_dict[key] = to_dict(value)
            elif isinstance(value, list) or isinstance(value, GeneratorType):
                new_dict[key] = _convert_list_of_fancy_hashes(value)
            else:
                new_dict[key] = value

    return new_dict


def to_json(obj):
    jsonable = None

    if isinstance(obj, list):
        jsonable = [ _datetimes_to_string(to_dict(changless_obj)) for changless_obj in obj ]
    else: 
        jsonable = _datetimes_to_string(to_dict(obj))

    return json.dumps(jsonable)

def _datetimes_to_string(obj):
    converted = obj
    for key, value in obj.items():
        if isinstance(value, dict):
            converted[key] = _datetimes_to_string(value)
        elif isinstance(value, list) or isinstance(value, GeneratorType):
            converted[key] = [ _datetimes_to_string(dict_obj) for dict_obj in value ]

        elif isinstance(value, datetime.datetime):
            converted[key] = value.isoformat()
            
    return converted 




from django.core.management import call_command
import logging


def reset_db(original_functional):
    def new_function(*args, **kwargs):
        call_command('flush',**{'interactive':False})
        original_functional(*args, **kwargs)
    return new_function()



from changeless.compare import fuzzyEquals
from changeless.types import FancyHash, FancyModel

import unittest
import os
from test_helpers import load_fixtures

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
from django.db.models.fields.related import ForeignKey, ManyToManyField
from django.contrib.auth.models import User
from django.test.simple import DjangoTestSuiteRunner

class ComparablesTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        test_runner = DjangoTestSuiteRunner(interactive=False, verbosity=1)
        test_db = test_runner.setup_databases()
        load_fixtures()


    def test_fuzzy_equals_with_model(self):
        user = User.objects.get(username='me')

        from_model = FancyModel(user)
        from_create_user = FancyHash( {'username':'me', 'email':'me@aol.com'})

        self.assertTrue(fuzzyEquals(from_create_user, from_model))


    def test_fuzzy_equal_with_nested_converted_like_immutable(self):
        i_obj = FancyHash({"name":'test name', 'sub_dict':{'name':'sub name', 'attrib':'sub attr value'}})
        second_i_obj = FancyHash({"name":'test name', 'sub_dict':{'name':'sub name', 'attrib':'sub attr value' }})
        self.assertTrue( fuzzyEquals(
                i_obj,
                second_i_obj ))

    def test_fuzzy_equal_with_nested_disjoined_converted_like_immutable(self):
        i_obj = FancyHash({"name":'test name', 'sub_dict':{'name':'sub name', 'attrib':'sub attr value'}})
        second_i_obj = FancyHash({"name":'test name', 'sub_dict':{'name':'sub name', 'attrib':'sub attr value', 'disjoined':"test value" }})
        self.assertTrue( fuzzyEquals(
                i_obj,
                second_i_obj ))

    def test_fuzzy_equal_create_with_converted_m2m_like_immutable(self):
        obj_with_relationship = FancyHash({"relationship":[{"attribute":"testValue"}]})
        obj_with_relationship2 = FancyHash({"relationship":[{"attribute":"testValue"}]})
        self.assertTrue(fuzzyEquals(
            obj_with_relationship,
            obj_with_relationship2))

    def test_fuzzy_equal_with_many_to_many_is_false_with_empty_list(self):
        obj_empty = FancyHash({"relationship":[]})
        obj_with_stuff = FancyHash({"relationship":[{"attribute":"testValue"}]})
        self.assertFalse(fuzzyEquals(
                obj_empty,
                obj_with_stuff ))

    def test_fuzzy_equal_with_many_to_many_is_false_with_value(self):
        obj_empty = FancyHash({"relationship":[{"attribute":"testValue not equal"}]})
        obj_with_stuff = FancyHash({"relationship":[{"attribute":"testValue"}]})
        self.assertFalse(fuzzyEquals(
                obj_empty,
                obj_with_stuff ))

    def test_fuzzy_equal_with_many_to_many_is_true(self):
        obj_with_stuff = FancyHash({"relationship":[{"attribute":"testValue"}]})
        obj_with_stuff_again = FancyHash({"relationship":[{"attribute":"testValue"}]})
        self.assertTrue(fuzzyEquals(
                obj_with_stuff, 
                obj_with_stuff_again ))



from changeless.methods import to_dict, _r_sub_dict

'''returns true if the union of dicts are the same minus the key in the ignore list'''
def fuzzyEquals(obj, other_obj, ignore=[]):
    obj = to_dict(obj)
    other_obj =to_dict(other_obj)

    refined_obj = _r_sub_dict(other_obj, obj)
    refined_other = _r_sub_dict(obj, other_obj )

    if refined_obj != refined_other:
        pass

    return refined_obj == refined_other 



import os
os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

from django.db.models.fields.related import ForeignKey, ManyToManyField
from django.contrib.auth.models import User
from django.test.simple import DjangoTestSuiteRunner

from changeless.test.myapp.models import Library, Book, Address

def load_fixtures():
    the_place_to_be = Address.objects.create(street_address="THE place")

    our_library = Library.objects.create(name="our library", address=the_place_to_be)

    reader1 = User.objects.create_user(
            'me',
            'me@aol.com',
            'johnpassword')
    reader2 = User.objects.create_user(
            'you',
            'you@gmail.com',
            'johnpassword')
    john = User.objects.create_user(
            'john',
            'lennon@thebeatles.com',
            'johnpassword')

    jacob = User.objects.create_user(
            'jacob',
            'lennon@thebeatles.com',
            'johnpassword')

    jingleheimer = User.objects.create_user(
            'schmidt',
            'not@thebeatles.com',
            'johnpassword')

    a_tail = Book.objects.create(
            title="A Tale of Two Cities",
            author=john,
            location=our_library)

    a_tail.readers.add(reader1)
    a_tail.readers.add(reader2)

    a_tail.save()

    Book.objects.create(
            title="A Tale of One Cities",
            author=john,
            location=our_library)

    Book.objects.create(
            title="A Tale of no Cities",
            author=jingleheimer,
            location=our_library)



from changeless.decorators import fancy_list, immutable_list, fancy_gen, immutable_gen 
from changeless.types import ImmutableHash, ImmutableModel, FancyHash, FancyModel


import unittest
import os
import types

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

from test_helpers import load_fixtures
from django.test.simple import DjangoTestSuiteRunner

from changeless.test.myapp.models import Library, Book, Address

class TestDecorators(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_runner = DjangoTestSuiteRunner(interactive=False, verbosity=1)
        test_db = test_runner.setup_databases()
        load_fixtures()

    def test_fancy_list_returns_correct_types(self):
        @fancy_list
        def get_books():
            return Book.objects.all()

        fancy_book_list = get_books()

        self.assertIsInstance(fancy_book_list, list)
        self.assertEqual(len(fancy_book_list) , 3)
        self.assertIsInstance(fancy_book_list[0], FancyModel)

    def test_fancy_list_object_gets_attributes(self):
        @fancy_list
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_list = get_cities()
        self.assertEqual(fancy_book_list[0].title , "A Tale of Two Cities")

    def test_immutable_list_returns_correct_types(self):
        @immutable_list
        def get_books():
            return Book.objects.all()

        immutable_book_list = get_books()

        self.assertIsInstance(immutable_book_list, list)
        self.assertEqual(len(immutable_book_list ) , 3)
        self.assertIsInstance(immutable_book_list[0], ImmutableModel)

    def test_immutable_list_object_gets_attributes(self):
        @immutable_list
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        immutable_book_list = get_cities()

        self.assertEqual(immutable_book_list[0].title , "A Tale of Two Cities")

    def test_fancy_list_should_be_able_to_submit_depth(self):
        @fancy_list(depth=1)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_list = get_cities()
        self.assertEqual(fancy_book_list[0].title , "A Tale of Two Cities")

    def test_fancy_list_depth_attribte_should_effect_depth(self):
        @fancy_list(depth=0)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_list = get_cities()

        with self.assertRaises(AttributeError):
            fancy_book_list[0].readers


    def test_immutable_list_should_be_able_to_submit_depth(self):
        @immutable_list(depth=1)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        immutable_book_list = get_cities()
        self.assertEqual(immutable_book_list[0].title , "A Tale of Two Cities")

    def test_immutable_list_depth_attrubte_should_effect_depth(self):

        @immutable_list(depth=0)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        immutable_book_list = get_cities()

        with self.assertRaises(AttributeError):
            immutable_book_list[0].readers

    def test_fancy_gen_should_return_a_generator(self):
        @fancy_gen
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_list = get_cities()
        self.assertIsInstance(fancy_book_list, types.GeneratorType)


    def test_fancy_gen_should_be_able_to_submit_depth(self):
        @fancy_gen(depth=1)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_generator = get_cities()
        fancy_book_list = [ book for book in fancy_book_generator ]

        self.assertEqual(fancy_book_list[0].title , "A Tale of Two Cities")

    def test_fancy_gen_depth_attrubte_should_effect_depth(self):

        @fancy_gen(depth=0)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_generator = get_cities()
        fancy_book_list = [ book for book in fancy_book_generator ]

        with self.assertRaises(AttributeError):
            fancy_book_list[0].readers

    def test_immutable_gen_should_return_a_generator(self):
        @immutable_gen
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        immutable_book_generator = get_cities()
        self.assertIsInstance(immutable_book_generator, types.GeneratorType)

    def test_immutable_gen_should_be_able_to_submit_depth(self):
        @immutable_gen(depth=1)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_generator = get_cities()
        fancy_book_list = [ book for book in fancy_book_generator ]

        self.assertEqual(fancy_book_list[0].title , "A Tale of Two Cities")

    def test_immutable_gen_depth_attrubte_should_effect_depth(self):

        @immutable_gen(depth=0)
        def get_cities():
            return Book.objects.filter(title="A Tale of Two Cities")

        fancy_book_generator = get_cities()
        fancy_book_list = [ book for book in fancy_book_generator ]

        with self.assertRaises(AttributeError):
            fancy_book_list[0].readers

    




import unittest
import json
import datetime
import types
import os

from changeless.types import FancyHash
from changeless.methods import to_json, to_dict
from changeless.decorators import fancy_gen

from changeless.test.myapp.models import Library, Book, Address

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

from test_helpers import load_fixtures
from django.test.simple import DjangoTestSuiteRunner

class TestConverters(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        test_runner = DjangoTestSuiteRunner(interactive=False, verbosity=1)
        test_db = test_runner.setup_databases()
        load_fixtures()


    def test_to_dict(self):
        test_object = FancyHash({"name":"test_name"})
        back_to_dict = to_dict(test_object)

        self.assertEqual( back_to_dict['name'], "test_name")

    def test_to_json(self):
        test_object = FancyHash({"name":"test_name"})
        test_json = to_json(test_object) 
        back_to_dict = json.loads(test_json)


        self.assertEqual( back_to_dict['name'], "test_name")

    def test_to_json_with_nested_attribute(self):
        test_object = FancyHash({"name":"test_name", "attribute":{"attr":"test_attr"}})
        test_json = to_json(test_object) 
        back_to_dict = json.loads(test_json)


        self.assertEqual( back_to_dict['attribute']['attr'], "test_attr")


    def test_to_json_with_nested_dict(self):
        test_object = FancyHash({"name":"test_name", "attribute":{"attr":"test_attr"}})
        test_json = to_json(test_object) 
        back_to_dict = json.loads(test_json)

        self.assertEqual( back_to_dict['attribute']['attr'], "test_attr")

    def test_to_json_with_list(self):
        test_fancy_list = [ 
                FancyHash({"name":"test_name", "thing":"value"}), 
                FancyHash({"name":"test_name2", "thing":"value2"}), 
                ]

        test_json = to_json(test_fancy_list)
        back_to_list = json.loads(test_json)
        self.assertEqual(back_to_list[0]['name'], "test_name")

    def test_to_json_converts_datetime(self):
        test_object = FancyHash({"name":"test", "time":datetime.datetime.now()})
        test_json = to_json(test_object) 
        back_to_dict = json.loads(test_json)

        self.assertIsInstance(back_to_dict['time'], types.UnicodeType)

    def test_to_json_converts_datetime_in_list(self):
        test_object = [FancyHash({"name":"test", "time":datetime.datetime.now()})]
        test_json = to_json(test_object) 
        back_to_dict = json.loads(test_json)

        self.assertIsInstance(back_to_dict[0]['time'], types.UnicodeType)

    def test_to_json_should_convert_datetimes_in_lists(self):
        test_object = [FancyHash({"name":"test", "times":[{"the_time":datetime.datetime.now()}] })]
        test_json = to_json(test_object) 
        back_to_dict = json.loads(test_json)

        self.assertIsInstance(back_to_dict[0]['times'][0]['the_time'], types.UnicodeType)

    def test_to_dict_should_convert_generator(self):
        @fancy_gen
        def get_all_books():
            return Book.objects.filter(title="A Tale of Two Cities")

        book_gen = get_all_books()
        list_of_books_dicts = to_dict(book_gen)
        self.assertEqual(list_of_books_dicts[0]['title'], "A Tale of Two Cities")




from fancy import BaseFancy, FancyModel
from conversion_helpers import model_to_dict

class BaseImmutable(BaseFancy):
    def __setattr__(self, name, value):
        if name != "__dict__":
            raise Exception("Immutable Object can not be changed")

class ImmutableHash(BaseImmutable):
    pass

class ImmutableModel(FancyModel):

    def __setattr__(self, name, value):
        if name != "__dict__":
            raise Exception("Immutable Object can not be changed")
    #def __init__(self, converted_object):
    #    super(ImmutableModel, self).__init__(converted_object)
        #self.__dict__ = converted_object
    #def __init__(self, django_model, depth=1):
    #    temp_dict = model_to_dict(django_model, depth)
    #    print temp_dict
    #    self.__dict__ = dict_to_base_type(temp_dict, BaseImmutable)




from django.db.models.fields.related import ForeignKey, ManyToManyField

def base_converter(base_type, a_dict):
    return base_type.__class__(a_dict)

def model_converter(base_type, a_model, depth=1):
    temp_dict = model_to_dict(a_model, depth)
    return base_type.__class__(temp_dict)

def convert_list_to(the_list, base_type):
    return [ base_type.__class__(an_item) for an_item in the_list ]


def dict_to_base_type(a_dict, base_type):
    converted_dict = {}
    for key, value in a_dict.items():
        if isinstance(value, dict):
            converted_dict[key] = base_type.__class__(value)
        elif isinstance(value, list):
            converted_dict[key] = convert_list_to(value, base_type)
        else:
            converted_dict[key] = value

    return converted_dict 

#def dict_to_base_type(a_dict, base_type):
#    converted_dict = {}
#    for key, value in a_dict.items():
#        if isinstance(value, dict):
#            converted_dict[key] = base_type(value)
#        elif isinstance(value, list):
#            converted_dict[key] = convert_list_to(value, base_type)
#        else:
#            converted_dict[key] = value
#
#    return converted_dict 


def model_to_dict(django_model, depth=1):

    temp_dict = {}

    for field in django_model._meta.local_fields:
        field_value = getattr(django_model, field.name)

        if isinstance(field, ForeignKey) and field_value is not None:
            if depth > 0:
                temp_dict[field.name] = model_to_dict(field_value, depth - 1)
        else:
            temp_dict[field.name] = field_value

    if depth > 0:
        for field in django_model._meta.local_many_to_many:
            field_list = []
            for m2m_item in getattr(django_model, field.name).all():
                field_list.append(model_to_dict(m2m_item, depth -1))
            temp_dict[field.name] = field_list

    return temp_dict



from distutils.core import setup

setup(
    name='changeless',
    version='0.1.26',
    author='Matt ODonnell',
    author_email='odonnell004@gmail.com',
    packages=['changeless', 'changeless.test', 'changeless.types'],
    url='http://pypi.python.org/pypi/Changeless/',
    license='LICENSE.txt',
    description='Making Immutable and stateless data structures',
    long_description=open('README.txt').read(),
    install_requires=[],
)
